Option Explicit                 ' Variables must be declared explicitly



'******************************************************************************
' Installer defaults.
'******************************************************************************
const AppName     = "S320"
const Version     = "7.00"
const Installer   = "S320 Installer"
const SpaceNeeded = 30                  ' Free space needed on drive in MB
const ExeFile     = "s320.exe"          ' Name of the executable
const AllUsers    = true                ' Install for user only
const Shortcut    = true                ' Create shortcut on desktop



'******************************************************************************
' Global constants and variables
'******************************************************************************
dim Tab, NewLine, Quote         ' String constants
dim Shell, FSO                  ' Global objects
dim ProgArgs                    ' Program arguments
dim Dbg                         ' Output debugging stuff
dim DoProgress                  ' Enable/disable progress bar
dim Language                    ' Program language
dim WindowsVersion              ' Version of operating system
dim SystemDrive                 ' The system drive
dim SystemRoot                  ' The windows directory
dim UserName                    ' Name if the current user
dim UserProfile                 ' User profile directory
dim ProgramFiles                ' Program files directory
dim AppData                     ' Application data directory
dim TetratecPrograms            ' Tetratec subfolder of the programs folder
dim InstallSource               ' Installation source directory
dim InstallTarget               ' Installation target directory
dim UninstallCtrlFileName       ' Name of the control file for the uninstaller
dim UninstallCtrlFile           ' Control file for the uninstaller
dim Uninstaller                 ' Path to the uninstaller file
dim UninstallerCmdLine          ' Command line for the uninstaller
dim Programs                    ' "Programs" menu folder
dim Desktop                     ' "Desktop" menu folder
dim RegUninstall                ' Registry key for uninstall entries
dim RegClasses                  ' Registry key for file classes
dim IniFile                     ' Name if ini file
dim KeepIni                     ' Keep an existing ini file?
dim SharedIni                   ' Create a shared ini file?
dim HelpFile	    		' Name of help file
dim WhatsNewFile		' Name of whatsnew file



'******************************************************************************
' Visual Basic Constants
'******************************************************************************
const ForReading        = 1



'******************************************************************************
' Windows version numbers
'******************************************************************************
const WIN_2000          = "5.0"
const WIN_XP            = "5.1"
const WIN_2003R2        = "5.2"
const WIN_VISTA         = "6.0"
const WIN_7             = "6.1"



'******************************************************************************
' Return a string in quotes
'******************************************************************************
function QuoteStr (Str)
    QuoteStr = Quote & Str & Quote
end function



'******************************************************************************
' Display an error message window with an OK button
'******************************************************************************
sub ErrorMsg (Msg)
    call MsgBox (Msg, vbOkOnly + vbExclamation, Installer)
end sub



'******************************************************************************
' Display an error message window and abort the installer
'******************************************************************************
sub Abort (Msg)
    call ErrorMsg (Msg)
    WScript.Quit (1)
end sub



'******************************************************************************
' Display a message with an OK button
'******************************************************************************
sub Message (Msg)
    call MsgBox (Msg, vbOkOnly + vbInformation, Installer)
end sub



'******************************************************************************
' Convert a number to a string
'******************************************************************************
function ToString (Num)
    ToString = FormatNumber (Num, vbFalse, vbTrue, vbFalse, vbFalse)
end function



'******************************************************************************
' Return a message in the current language
'******************************************************************************
function GetMsg (Key)
    dim Msg
    if Language = "de" then
        select case Key
            case "MSG_ABORTINFO"
                Msg = "Die Installation wurde abgebrochen."
            case "MSG_ADMIN"
                Msg = "Sie mssen Administrator sein, um %1 zu installieren."
                Msg = Msg & " Wollen Sie fortfahren?"
            case "MSG_CLOSEERR"
                Msg = "Fehler beim Schliessen der Datei" & NewLine & "%1:" & NewLine & "%2"
            case "MSG_CONTACTENTRY"
                Msg = "TetraTec kontaktieren"
            case "MSG_COPYERR"
                Msg = "Fehler beim Kopieren von %1 nach %2: " & NewLine & "%3"
            case "MSG_CREATEDIR"
                Msg = "Der Ordner %1 existiert nicht." & NewLine & "Soll er erzeugt werden?"
            case "MSG_CREATEERR"
                Msg = "%1 kann nicht erzeugt werden:" & NewLine & "%2"
            case "MSG_DELETEERR"
                Msg = "%1 kann nicht gelscht werden:" & NewLine & "%2"
            case "MSG_DRIVESPACE"
                Msg = "Nicht genug Platz auf Laufwerk %1" & NewLine
                Msg = Msg & "Mindestens %2 MB sind ntig."
            case "MSG_INSTALLPATH"
                Msg = "Die Installation erfolgt in %1"
            case "MSG_HELPENTRY"
                Msg = "%1 Hilfe"
            case "MSG_KEEPINI"
                Msg = "Es existiert bereits eine Datei mit Einstellungen." & NewLine
                Msg = Msg & "Soll sie bernommen werden?"
            case "MSG_MOVEERR"
                Msg = "Fehler beim Umbenennen von %1 nach %2: " & NewLine & "%3"
            case "MSG_OPENERR"
                Msg = "%1 kann nicht geffnet werden:" & NewLine & "%2"
            case "MSG_REGWRITEERR"
                Msg = "Fehler bei der Installation: Die Registrierungseintrge "
                Msg = Msg & " konnten nicht geschrieben werden."
            case "MSG_REMOVEENTRY"
                Msg = "%1 entfernen"
            case "MSG_REMOVEDIR"
                Msg = "Ein Ordner mit dem Namen %1 existiert bereits."
                Msg = Msg & " Soll er entfernt werden?"
            case "MSG_REMOVEOLD"
                Msg = "Es existiert bereits eine alte Version. Soll diese"
                Msg = Msg & " entfernt werden?"
            case "MSG_SHARESETTINGS"
                Msg = "Sollen Einstellungen fr alle Benutzer gelten?"
            case "MSG_SUCCESS"
                Msg = "Die Installation war erfolgreich!"
            case "MSG_UNINSTALLERR"
                Msg = "Es trat ein Problem beim Entfernen der alten Version"
                Msg = Msg & " auf. Bitte deinstallieren Sie die alte Version"
                Msg = Msg & " und starten Sie die Installation neu."
            case "MSG_WHATSNEW"
                Msg = "Neuigkeiten zum Controller S320"
            case "MSG_WRITEERR"
                Msg = "Fehler beim Schreiben von %1:" & NewLine & "%2"
            case else
                Msg = Key
        end select
    end if

    if IsEmpty (Msg) then
        ' No assignment, use english
        select case Key
            case "MSG_ABORTINFO"
                Msg = "Installation was aborted."
            case "MSG_ADMIN"
                Msg = "You must be Administrator to install %1."
                Msg = Msg & " Are you sure you want to continue?"
            case "MSG_CLOSEERR"
                Msg = "Error closing file %1:" & NewLine & "%2"
            case "MSG_CONTACTENTRY"
                Msg = "How to Contact TetraTec"
            case "MSG_COPYERR"
                Msg = "Cannot copy %1 to %2: " & NewLine & "%3"
            case "MSG_CREATEDIR"
                Msg = "%1 does not exist." & NewLine & "Create it?"
            case "MSG_CREATEERR"
                Msg = "Cannot create %1:" & NewLine & "%2"
            case "MSG_DELETEERR"
                Msg = "Cannot delete %1:" & NewLine & "%2"
            case "MSG_DRIVESPACE"
                Msg = "Not enough space left on drive %1" & NewLine
                Msg = Msg & "At least %2 MB are needed."
            case "MSG_INSTALLPATH"
                Msg = "The package will be installed in %1"
            case "MSG_HELPENTRY"
                Msg = "%1 Help"
            case "MSG_KEEPINI"
                Msg = "A file containing program settings does already exist." & NewLine
                Msg = Msg & "Do you want to keep the old settings?"
            case "MSG_MOVEERR"
                Msg = "Cannot rename %1 to %2: " & NewLine & "%3"
            case "MSG_OPENERR"
                Msg = "Cannot open %1:" & NewLine & "%2"
            case "MSG_REGWRITEERR"
                Msg = "Installation failed: Cannot write to the registry!"
            case "MSG_REMOVEENTRY"
                Msg = "Remove %1"
            case "MSG_REMOVEDIR"
                Msg = "A folder with the name %1 does already exist."
                Msg = Msg & " Is it ok to remove the folder?"
            case "MSG_REMOVEOLD"
                Msg = "Found an old version. Remove it?"
            case "MSG_SHARESETTINGS"
                Msg = "Do you want to share the settings between all users?"
            case "MSG_SUCCESS"
                Msg = "Installation was successful!"
            case "MSG_UNINSTALLERR"
                Msg = "There was a problem uninstalling the old version. Please"
                Msg = Msg & " uninstall the old program manually and restart"
                Msg = Msg & " the installation."
            case "MSG_WHATSNEW"
                Msg = "New Controller S320 features"
            case "MSG_WRITEERR"
                Msg = "Error writing to %1:" & NewLine & "%2"
            case else
                Msg = Key
        end select
    end if
    GetMsg = Msg
end function



'******************************************************************************
' Format a string replacing %n specifiers in the format string F
'******************************************************************************
function Fmt (F, Values)
    dim I, Count, Key, Val, Start, Pos
    Count = UBound (Values)     ' How many values?
    for I = Count to 0 step -1
        Key = "%" & ToString (I)
        select case VarType (Values (I))
            case vbEmpty
                Val = ""
            case vbInteger
                Val = ToString (Values (I))
            case vbLong
                Val = ToString (Values (I))
            case vbNull
                Val = ""
            case vbSingle
                Val = ToString (Values (I))
            case vbDouble
                Val = ToString (Values (I))
            case vbString
                Val = Values (I)
            case else
                Abort ("Internal error: Invalid conversion in Format()")
        end select
        F = Replace (F, Key, Val)
    next
    Fmt = F
end function



'******************************************************************************
' Format a message replacing %n specifiers in the format string F
'******************************************************************************
function FmtMsg (Msg, Values)
    FmtMsg = Fmt (GetMsg (Msg), Values)
end function



'******************************************************************************
' Return the windows version number
'******************************************************************************
function WinVersion ()
    Dim Service, OSList, Version, OS
    Version = ""

    on error resume next
    Set Service = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\cimv2")
    Set OSList = Service.ExecQuery ("Select * from Win32_OperatingSystem")
    For Each OS in OSList
        Version = OS.Version
        'Wscript.Echo objOperatingSystem.Caption & " " & objOperatingSystem.Version
    next
    if (Len (Version) > 3) then
        Version = Left (Version, 3)
    end if
    WinVersion = Version
    on error goto 0
end function



'******************************************************************************
' Return an environment string. Fix up Microsofts "innovative" ideas.
'******************************************************************************
function GetEnv (Key)
    dim Value
    Value = Shell.ExpandEnvironmentStrings (Key)
    if Value = Key then
        GetEnv = vbNullString
    else
        GetEnv = Value
    end if
end function



'******************************************************************************
' Build a path from two components
'******************************************************************************
function BuildPath (Path, Name)
    BuildPath = FSO.BuildPath (Path, Name)
end function



'******************************************************************************
' Return true if the file with the given name exists
'******************************************************************************
function FileExists (Name)
    On Error Resume Next
    FileExists = FSO.FileExists (Name)
end function



'******************************************************************************
' Return true if the folder with the given name exists
'******************************************************************************
function FolderExists (Name)
    On Error Resume Next
    FolderExists = FSO.FolderExists (Name)
end function



'******************************************************************************
' Copy a file and return an error message (empty string if no error)
'******************************************************************************
function CopyFile (Source, Target)
    if Right (Target, 1) <> "\" and FolderExists (Target) then
        Target = Target & "\"
    end if
    On Error Resume Next
    call FSO.CopyFile (Source, Target)
    CopyFile = Err.Description
    on error goto 0
end function



'******************************************************************************
' Move a file and return an error message (empty string if no error)
'******************************************************************************
function MoveFile (Source, Target)
    if Right (Target, 1) <> "\" and FolderExists (Target) then
        Target = Target & "\"
    end if
    On Error Resume Next
    call FSO.MoveFile (Source, Target)
    MoveFile = Err.Description
    on error goto 0
end function



'******************************************************************************
' Create a folder and all parent folders and return an error string
'******************************************************************************
function CreateFolder (Path)

    ' If the parent folder does not exist, try to create it
    dim ParentFolder
    ParentFolder = FSO.GetParentFolderName (Path)
    if ParentFolder <> "" and not FolderExists (ParentFolder) then
        CreateFolder (ParentFolder)
    end if

    ' Now try to create the actual folder
    On Error Resume Next
    FSO.CreateFolder (Path)
    CreateFolder = Err.Description

end function



'******************************************************************************
' Delete a file and return an error string
'******************************************************************************
function DeleteFile (Name)
    On Error Resume Next
    call FSO.DeleteFile (Name, true)
    DeleteFile = Err.Description
end function



'******************************************************************************
' Delete a folder and return an error string
'******************************************************************************
function DeleteFolder (Path)
    On Error Resume Next
    call FSO.DeleteFolder (Path, true)
    DeleteFolder = Err.Description
end function



'******************************************************************************
' Read a string from the registry. Return an empty string if nothing was found.
'******************************************************************************
function RegReadStr (Key)
    On Error Resume Next
    RegReadStr = Shell.RegRead (Key)
    if Err.Number <> 0 then
        RegReadStr = ""
    end if
end function



'******************************************************************************
' Write a binary value to the registry, return an error description
'******************************************************************************
function RegWriteBin (Key, Value)
    on error resume next
    Shell.RegWrite Key, Value, "REG_BINARY"
    RegWriteBin = Err.Description
    on error goto 0
    WriteUninstallCtrlFile ("R " & Key)
end function



'******************************************************************************
' Write a DWORD value to the registry, return an error description
'******************************************************************************
function RegWriteDWORD (Key, Value)
    on error resume next
    Shell.RegWrite Key, Value, "REG_DWORD"
    RegWriteDWORD = Err.Description
    on error goto 0
    WriteUninstallCtrlFile ("R " & Key)
end function



'******************************************************************************
' Write a string value to the registry, return an error description
'******************************************************************************
function RegWriteStr (Key, Value)
    on error resume next
    Shell.RegWrite Key, Value, "REG_SZ"
    RegWriteStr = Err.Description
    on error goto 0
    WriteUninstallCtrlFile ("R " & Key)
end function



'******************************************************************************
' Run a program, wait for its termination and return an error code.
'******************************************************************************
function Run (Cmd, WinState)
    dim ErrCode

    On Error Resume Next
    ErrCode = Shell.Run (Cmd, WinState, true)
    if Err.Number <> 0 then
        ErrCode = Err.Number
    end if
    Run = ErrCode
end function



'******************************************************************************
' Display a progress bar using the internet exploder
'******************************************************************************
dim PBDoc               ' Progress bar document object
dim PBVal               ' Current progress bar setting
dim IEApp               ' Internet exploder application object
set PBDoc = nothing
set IEApp = nothing
PBVal = -1

sub ProgressBar (Percent)

    ' Bail out early if the progress bar is switched off
    if not DoProgress then
        exit sub
    end if


    ' Remember the last setting
    PBVal = Percent

    'Create the status bar window
    if PBDoc is nothing then

        if ((Cint (Percent) < 0) or (Cint (Percent) > 100)) then
            ' Close request, but object already destroyed
            exit sub
        end if

        ' Create an object that controls the internet exploder
        set IEApp = CreateObject ("InternetExplorer.Application")

        ' Set the exploder to fullscreen and retrieve its size
        dim ScreenHeight, ScreenWidth
        IEApp.Visible    = false
        IEApp.FullScreen = true
        ScreenWidth      = IEApp.Width
        ScreenHeight     = IEApp.Height
        IEApp.FullScreen = false

        ' Now prepare the actual window
        IEApp.Offline    = true
        IEApp.AddressBar = false
        IEApp.Height     = 100
        IEApp.Width      = 250
        IEApp.MenuBar    = false
        IEApp.StatusBar  = false
        IEApp.Silent     = true
        IEApp.ToolBar    = false
        IEApp.Resizable  = false
        IEApp.Left       = (ScreenWidth  - IEApp.Width)  / 2
        IEApp.Top        = (ScreenHeight - IEApp.Height) / 2
        call IEApp.Navigate ("about:blank")
        do while IEApp.Busy
            call WScript.Sleep (100)
        loop

        ' Connect to the displayed document
        do until not PBDoc is nothing
            call WScript.Sleep (100)
            set PBDoc = IEApp.Document
        loop

        ' Generate a new document showing a status bar
        PBDoc.Open
        call PBDoc.Write ("<html><head><title>" & Installer & " status</title></head>")
        call PBDoc.Write ("<body bgcolor=#C0C0C0><center>")
        call PBDoc.Write ("<table width=100% border=1 frame=box><tr><td>")
        call PBDoc.Write ("<table id=status width=0 border=0 cellpadding=0 cellspacing=0 bgcolor=#FFFFFF>")
        call PBDoc.Write ("<tr><td>&nbsp</td></tr></table>")
        call PBDoc.Write ("</td></tr></table>")
        call PBDoc.Write ("</center></body></html>")
        PBDoc.Close

        ' Display the exploder window
        IEApp.Visible = true

    else
        if ((Cint (Percent) < 0) or (Cint (Percent) > 100)) then
            ' Request for shutdown
            IEApp.Visible = false
            set PBDoc = nothing
            IEApp.Quit
            set IEApp = nothing
        else
            ' Update the status bar
            if Cint (Percent) = 0 then
                PBDoc.all.status.width = "1%"
                PBDoc.all.status.bgcolor = "#C0C0C0"
            else
                PBDoc.all.status.width = Cstr (Cint (Percent)) & "%"
                PBDoc.all.status.bgcolor = "#0000C0"
            end if
        end if
    end if
end sub



'******************************************************************************
' Initialize global variables
'******************************************************************************
sub InitializeGlobals ()
    dim I

    ' String stuff used for formatting
    Tab     = Chr (9)
    NewLine = Chr (13)
    Quote   = Chr (34)

    ' Global objects
    set Shell = WScript.CreateObject ("WScript.Shell")
    set FSO   = CreateObject ("Scripting.FileSystemObject")

    ' Arguments
    set ProgArgs = WScript.Arguments

    ' Handle program arguments
    Dbg = false
    DoProgress = true
    Language = "de"
    for I = 0 to ProgArgs.Count-1
        select case ProgArgs(I)
            case "-de"
                Language = "de"
            case "-debug"
                Dbg = true
            case "-en"
                Language = "en"
            case "-np"
                DoProgress = false
        end select
    next

    ' Windows version
    WindowsVersion = WinVersion ()

    ' Paths and locations
    SystemDrive = GetEnv ("%SystemDrive%")
    if SystemDrive = vbNullString then
        SystemDrive = "c:"
    end if
    SystemRoot = GetEnv ("%SystemRoot%")
    if SystemRoot = vbNullString then
        SystemRoot = BuildPath (SystemDrive, "winnt")
    end if
    UserName = GetEnv ("%USERNAME%")
    if UserName = vbNullString then
        UserName = "Administrator"
    end if
    UserProfile = GetEnv ("%USERPROFILE%")
    if UserProfile = vbNullString then
        UserProfile = BuildPath (SystemDrive, "Dokumente und Einstellungen\" & UserName)
    end if
    ProgramFiles = GetEnv ("%ProgramFiles(x86)%")
    if ProgramFiles = vbNullString then
        ProgramFiles = GetEnv ("%ProgramFiles%")
        if ProgramFiles = vbNullString then
            ProgramFiles = BuildPath (SystemDrive, "Programme")
        end if
    end if
    AppData = GetEnv ("%AppData%")
    if AppData = vbNullString then
        AppData = UserProfile
    end if
    TetratecPrograms = BuildPath (ProgramFiles, "TetraTec")
    InstallSource = FSO.GetParentFolderName (WScript.ScriptFullName)
    InstallTarget = BuildPath (TetratecPrograms, AppName)
    if AllUsers then
        Programs = Shell.SpecialFolders ("AllUsersPrograms")
        Desktop  = Shell.SpecialFolders ("AllUsersDesktop")
    else
        Programs = Shell.SpecialFolders ("Programs")
        Desktop  = Shell.SpecialFolders ("Desktop")
    end if

    ' Uninstaller
    set UninstallCtrlFile = nothing
    Uninstaller = BuildPath (TetratecPrograms, "uninstall.vbs")
    UninstallCtrlFileName = BuildPath (InstallTarget, "uninstall.lst")
    UninstallerCmdLine = "-" & Language & " " & AppName & " """ & UninstallCtrlFileName & """"

    ' Registry paths
    RegUninstall = "HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\" & AppName & "\"
    RegClasses   = "HKLM\Software\Classes\"

    ' Files
    IniFile     = BuildPath (InstallTarget, "s320term.ini")
    if Language = "de" then
	HelpFile = "s320hlpd.chm"
	WhatsNewFile = "whatsnew\whatsnewd.html"
    else
	HelpFile = "s320hlpe.chm"
	WhatsNewFile = "whatsnew\whatsnewe.html"
    end if
end sub



'******************************************************************************
' Ask a yes/no question and return the result. "Yes" is default.
'******************************************************************************
function AskYesNo (Question)
    AskYesNo = MsgBox (Question, vbYesNo + vbQuestion + vbDefaultButton1, Installer)
end function



'******************************************************************************
' Ask a yes/no question and return the result. "No" is default.
'******************************************************************************
function AskNoYes (Question)
    AskNoYes = MsgBox (Question, vbYesNo + vbQuestion + vbDefaultButton2, Installer)
end function



'******************************************************************************
' Tell the user that the installation was aborted and terminate the script
'******************************************************************************
sub InfoAbort ()
    call MsgBox (GetMsg ("MSG_ABORTINFO"), vbOkOnly + vbInformation, Installer)
    WScript.Quit (0)
end sub



'******************************************************************************
' Input routine with the window caption preset
'******************************************************************************
function Input (Prompt, Default)
    Input = InputBox (Prompt, Installer, Default)
end function



'******************************************************************************
' Function that displays the paths and locations found
'******************************************************************************
function OneLoc (Key, Value)
    dim Result
    Result = Trim (Key)
    if Len (Result) <= 8 then
        Result = Result & Tab
    end if
    OneLoc = Result & Tab & "=" & Tab & Value & NewLine
end function

sub ShowPathsAndLocations ()
    dim Msg
    Msg = Msg & OneLoc ("SystemDrive",   SystemDrive)
    Msg = Msg & OneLoc ("SystemRoot",    SystemRoot)
    Msg = Msg & OneLoc ("UserName",      UserName)
    Msg = Msg & OneLoc ("UserProfile",   UserProfile)
    Msg = Msg & OneLoc ("ProgramFiles",  ProgramFiles)
    Msg = Msg & OneLoc ("AppData",       AppData)
    Msg = Msg & OneLoc ("InstallSource", InstallSource)
    Msg = Msg & OneLoc ("InstallTarget", InstallTarget)
    Msg = Msg & OneLoc ("Programs",      Programs)
    Msg = Msg & OneLoc ("Desktop",       Desktop)
    Msg = Msg & OneLoc ("Free space",    ToString (GetDriveSpace (InstallTarget)))

    call MsgBox (Msg, vbOkOnly, "Paths and Locations")
end sub



'******************************************************************************
' Return the amount of free space for a path (in Megabytes)
'******************************************************************************
function GetDriveSpace (Path)
    dim Drive
    On Error Resume Next
    set Drive = FSO.GetDrive (FSO.GetDriveName (Path))
    if Err.Number <> 0 then
        GetDriveSpace = 0
    else
        GetDriveSpace = Drive.FreeSpace / (1024 * 1024)
    end if
end function



'******************************************************************************
' Check that were running this script as admin
'******************************************************************************
sub CheckAdminRights ()

    ' FIXME: This check is not perfect
    if UserName <> "Administrator" then
        dim Args(1)
        Args(1) = AppName

        if AskNoYes (FmtMsg ("MSG_ADMIN", Args)) <> vbYes then
            WScript.Quit (1)
        end if
    end if

end sub



'******************************************************************************
' Remove an old installation.
'******************************************************************************
sub RemoveOldInstall (UninstallCmd)

    dim ErrCode

    ' Execute the uninstall
    ErrCode = Run (UninstallCmd, 0)

    ' Tell the user that the uninstall is done
    if ErrCode <> 0 then
        call Abort (GetMsg ("MSG_UNINSTALLERR"))
    end if
end sub



'******************************************************************************
' Check if there is an old installation. Offer to remove it.
'******************************************************************************
sub CheckOldInstall ()

    dim UninstallCmd

    ' Read the uninstall command from the registry
    UninstallCmd = RegReadStr (RegUninstall & "UninstallString")

    ' Check if there is already an executable
    if UninstallCmd <> "" then

        ' Ask to remove an old install
        if AskYesNo (GetMsg ("MSG_REMOVEOLD")) = vbYes then
            ' Remove the old installation
            call RemoveOldInstall (UninstallCmd)
        end if

    end if

end sub



'******************************************************************************
' Check that the install target exists. Offer to create it.
'******************************************************************************
sub CheckInstallTarget ()

    dim Msg, Result, Args(2)

    ' Tell the user about the install target and ask if it's ok
    Args(1) = InstallTarget
    Msg = FmtMsg ("MSG_INSTALLPATH", Args)
    if MsgBox (Msg, vbOkCancel, Installer) <> vbOk then
        call InfoAbort ()
    end if

    ' Check if there's enough space left on the target drive
    if GetDriveSpace (InstallTarget) < SpaceNeeded then
        Args(1) = FSO.GetDriveName (InstallTarget)
        Args(2) = SpaceNeeded
        call Abort (FmtMsg ("MSG_DRIVESPACE", Args))
    end if

    ' Check if the install path exist, create it if necessary
    if not FolderExists (InstallTarget) then
        Result = CreateFolder (InstallTarget)
        if Result <> "" then
            Args(1) = InstallTarget
            Args(2) = Result
            call Abort (FmtMsg ("MSG_CREATEERR", Args))
        end if
    end if

    ' If we have an existing INI file in the target directory, ask the user
    ' if he wants to keep it. Otherwise ask if he wants to have a shared ini
    ' file. Do all this here, so the installation can run through later.
    KeepIni = false
    if FileExists (IniFile) then
        if AskYesNo (GetMsg ("MSG_KEEPINI")) <> vbYes then
            Result = DeleteFile (IniFile)
            if Result <> "" then
                Args(1) = IniFile
                Args(2) = Result
                ErrorMsg (FmtMsg ("MSG_DELETEERR", Args))
                KeepIni = true          ' Just keep it
            end if
        else
            KeepIni = true
        end if
    end if
    if not KeepIni then
        SharedIni = AskNoYes (GetMsg ("MSG_SHARESETTINGS")) = vbYes
    else
        SharedIni = false
    end if

end sub



'******************************************************************************
' Create the uninstall control file
'******************************************************************************
sub CreateUninstallCtrlFile ()

    dim Filename

    ' Generate the filename
    on Error resume next
    set UninstallCtrlFile = FSO.CreateTextFile (UninstallCtrlFileName, true)
    if Err.Number <> 0 then
        dim Args(2)
        Args(1) = UninstallCtrlFileName
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_CREATEERR", Args))
        WScript.Quit (1)
    end if

    ' Write the name of the file itself to the file
    call WriteUninstallCtrlFile ("F " & UninstallCtrlFileName)

end sub



'******************************************************************************
' Write to the uninstall control file
'******************************************************************************
sub WriteUninstallCtrlFile (Line)

    on error resume next
    UninstallCtrlFile.WriteLine (Line)
    if Err.Number <> 0 then
        dim Args(2)
        Args(1) = UninstallCtrlFileName
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_WRITEERR", Args))
        WScript.Quit (1)
    end if

end sub



'******************************************************************************
' Close the uninstall control file
'******************************************************************************
sub CloseUninstallCtrlFile ()

    on error resume next
    UninstallCtrlFile.Close
    if Err.Number <> 0 then
        dim Args(2)
        Args(1) = UninstallCtrlFileName
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_CLOSEERR", Args))
        WScript.Quit (1)
    end if

end sub



'******************************************************************************
' Copy the application files
'******************************************************************************
sub RecursiveCopy (Dir, SourcePath, TargetPath)

    dim File, TargetFile, SubDir, SourceName, TargetName, Result, Args(3)

    ' Copy all files in this folder
    for each File in Dir.Files

        ' Generate source and target file names
        SourceName = BuildPath (SourcePath, File.Name)
        TargetName = BuildPath (TargetPath, File.Name)

        ' Copy the file. The error check doesn't seem to work.
        on error resume next
        File.Copy (TargetName)
        if Err.Number <> 0 then
            Args(1) = SourceName
            Args(2) = TargetName
            Args(3) = Err.Description
            call ErrorMsg (FmtMsg ("MSG_COPYERR", Args))
            call AbortInfo ()
        end if
        on error goto 0

        ' Remove the r/o attribute from the target file if set
        set TargetFile = FSO.GetFile (TargetName)
        if TargetFile.Attributes mod 2 = 1 then
            TargetFile.Attributes = TargetFile.Attributes - 1
        end if

        ' Remember this file in the uninstall control file
        call WriteUninstallCtrlFile ("F " & TargetName)
    next

    ' Handle all subdirectories
    for each SubDir in Dir.SubFolders

        ' Update the progress bar with each copied directory
        if PBVal <= 62 then
            call ProgressBar (PBVal + 8)
        end if

        ' Generate the new directory names
        SourceName = BuildPath (SourcePath, SubDir.Name)
        TargetName = BuildPath (TargetPath, SubDir.Name)

        ' Generate the new target dir. Notify the user about errors, but
        ' otherwise ignore them.
        Result = CreateFolder (TargetName)
        if Result <> "" then
            ' Display an error but try to continue
            Args(1) = TargetName
            Args(2) = Result
            call ErrorMsg (FmtMsg ("MSG_CREATEERR", Args))
        end if

        ' Recursively process files in the subdirectory
        call RecursiveCopy (SubDir, SourceName, TargetName)

        ' Remember the subdirectory in the uninstall control file
        WriteUninstallCtrlFile ("D " & TargetName)

    next
end sub

sub CopyFiles ()

    ' Update the progress bar
    call ProgressBar (10)

    ' Copy all files generating entries in the uninstall control file
    call RecursiveCopy (FSO.GetFolder (InstallSource), InstallSource, InstallTarget)

    ' Update the progress bar
    call ProgressBar (70)
end sub



'******************************************************************************
' Determine the version number of an uninstaller
'******************************************************************************
function UninstallerVersion (Name)

    const ForReading = 1

    dim File, Line
    dim RE, Match

    ' Assume we could not determine the version
    UninstallerVersion = 0

    ' Open the file. Checking Err doesn't work here, don't know why.
    set File = nothing
    on error resume next
    set File = FSO.OpenTextFile (Name, ForReading)
    on error goto 0
    if File is nothing then
        ' No such file
        exit function
    end if

    ' Setup the regular expression matcher
    set RE = new RegExp
    RE.Pattern = "^\s*const\s+UninstallerVersion\s*=\s*(\d+).*$"
    RE.IgnoreCase = true
    RE.Global = true

    ' Read all lines and check for a version declaration
    do while File.AtEndOfStream <> true

        ' Read the next line
        on error resume next
        Line = File.ReadLine
        on error goto 0

        ' Check if it contains what we are looking for. While the Execute
        ' method does also work, it is not possible to access the submatches
        ' of the regular expression. The examples from the reference guide
        ' are VB (not VBS) code, and do not work.
        Match = RE.Replace (Line, "$1")
        if Match <> Line then
            UninstallerVersion = CInt (Match)
            exit do
        end if
    loop

    ' Close the file
    on error resume next
    call File.Close ()
    on error goto 0

end function

'******************************************************************************
' Copy the uninstaller if there is no newer version
'******************************************************************************
sub CopyUninstaller ()

    dim UninstallerSource, Result, Args(3)
    dim OldVer, NewVer

    ' Path of the uninstaller
    UninstallerSource = BuildPath (InstallTarget, "uninstall.vbs")

    ' Check if there is an old uninstaller
    if FileExists (Uninstaller) then

        ' Determine the versions of the old and new uninstaller
        OldVer = UninstallerVersion (Uninstaller)
        NewVer = UninstallerVersion (UninstallerSource)

        ' Bail out if our new version is actually older or same
        if NewVer <= OldVer then
            exit sub
        end if

    end if

    ' Copy the uninstaller into the tetratec directory
    Result = CopyFile (UninstallerSource, Uninstaller)
    if Result <> "" then
        Args(1) = UninstallerSource
        Args(2) = Uninstaller
        Args(3) = Result
        call ErrorMsg (FmtMsg ("MSG_COPYERR", Args))
    end if

end sub



'******************************************************************************
' Create the registry entries
'******************************************************************************
sub CreateProgAssign (Name, Desc, Ext, Cmd)

    ' Assign a name to a file extension
    RegWriteStr RegClasses & Ext & "\", Name

    ' Assign a command to the name. The RegWriteStr function will automatically
    ' allocate necessary intermediate levels, but RegDelete cannot delete them.
    ' So enter these additional levels into the uninstall control file for the
    ' uninstaller.
    RegWriteStr RegClasses & Name & "\", Desc
    RegWriteStr RegClasses & Name & "\shell\open\command\", Cmd
    call WriteUninstallCtrlFile ("R " & RegClasses & Name & "\shell\")
    call WriteUninstallCtrlFile ("R " & RegClasses & Name & "\shell\open\")
end sub

sub CreateRegEntries ()

    dim Cmd

    ' Create the entry in Systemsteuerung -> Software. Check if the first write
    ' succeeds. If not, we don't have admin rights.
    if RegWriteBin (RegUninstall, 1) <> "" then
        call Abort (GetMsg ("MSG_REGWRITEERR"))
    end if
    call RegWriteStr (RegUninstall & "DisplayName", AppName & " (" & Version & ")")
    call RegWriteStr (RegUninstall & "DisplayIcon", QuoteStr (BuildPath (InstallTarget, ExeFile)))
    call RegWriteStr (RegUninstall & "DisplayVersion", Version)
    call RegWriteDWORD (RegUninstall & "NoModify", 1)
    call RegWriteDWORD (RegUninstall & "NoRepair", 1)
    call RegWriteStr (RegUninstall & "Publisher", "TetraTec Instruments GmbH")
    call RegWriteStr (RegUninstall & "UninstallString", "wscript //nologo """ & Uninstaller & """ " & UninstallerCmdLine)
    call RegWriteStr (RegUninstall & "URLInfoAbout", "http://www.tetratec.de/")

    ' Assign a program to .spx files
    Cmd = BuildPath (InstallTarget, ExeFile) & " -spx " & QuoteStr ("%1")
    call CreateProgAssign ("S320Executable", "S320 Executable File", ".spx", Cmd)

    ' Assign a program to .spc files
    Cmd = BuildPath (InstallTarget, ExeFile) & " -spc " & QuoteStr ("%1")
    call CreateProgAssign ("S320Project", "S320 Project File", ".spc", Cmd)

end sub



'******************************************************************************
' Function that creates an URL
'******************************************************************************
sub CreateUrl (Name, Url, Description)
    ' Ignore errors
    On Error Resume Next

    dim Link
    set Link = Shell.CreateShortcut (Name)
    Link.TargetPath = Url
    Link.Description = Description
    Link.Save

    ' Allow errors again
    on error goto 0

    ' Write the file name to the uninstall control file
    WriteUninstallCtrlFile ("F " & Name)
end sub



'******************************************************************************
' Function that creates a shortcut
'******************************************************************************
sub CreateShortcut (Name, Exe, Args, Description)
    ' Ignore errors
    On Error Resume Next

    dim Link
    set Link = Shell.CreateShortcut (Name)
    Link.TargetPath  = Exe
    Link.Arguments   = Args
    Link.WindowStyle = 1
    Link.Description = Description
    Link.WorkingDirectory = InstallTarget
    Link.Save

    ' Allow errors again
    on error goto 0

    ' Write the file name to the uninstall control file
    WriteUninstallCtrlFile ("F " & Name)
end sub



'******************************************************************************
' Function that creates an application shortcut
'******************************************************************************
sub CreateAppShortcut (Path)
    dim Name, Exe
    Name = BuildPath (Path, AppName & ".lnk")
    Exe  = BuildPath (InstallTarget, ExeFile)
    call CreateShortcut (Name, Exe, "", AppName)
end sub



'******************************************************************************
' Function that creates the menu entries
'******************************************************************************
sub CreateMenuEntries ()
    dim Folder, Result, Name, Desc, Exe, Args(2)

    ' Create the parent and program folders. Ignore errors when creating the
    ' parent folder, since it may already exist.
    Folder = BuildPath (Programs, "TetraTec\" & AppName)
    Result = CreateFolder (Folder)
    if Result <> "" then
        ' Display an error but try to continue
        Args(1) = Folder
        Args(2) = Result
        call ErrorMsg (FmtMsg ("MSG_CREATEERR", Args))
    end if

    ' Create the application shortcut in the menu folder
    CreateAppShortcut (Folder)

    ' Create an uninstall shortcut in the menu folder
    Args(1) = AppName
    Desc = FmtMsg ("MSG_REMOVEENTRY", Args)
    Name = BuildPath (Folder, Desc & ".lnk")
    call CreateShortcut (Name, Uninstaller, UninstallerCmdLine, Desc)

    ' Create a help shortcut in the menu folder
    Args(1) = AppName
    Desc = FmtMsg ("MSG_HELPENTRY", Args)
    Name = BuildPath (Folder, Desc & ".lnk")
    Exe  = BuildPath (InstallTarget, HelpFile)
    call CreateShortcut (Name, Exe, "", Desc)

    ' Create the "contacts" shortcut in the menu folder
    Desc = GetMsg ("MSG_CONTACTENTRY")
    Name = BuildPath (Folder, Desc & ".url")
    Exe  = "file://" & BuildPath (InstallTarget, "contact.html")
    call CreateUrl (Name, Exe, Desc)

    ' Create the "What's new" shortcut in the menu folder
    Desc = GetMsg ("MSG_WHATSNEW")
    Name = BuildPath (Folder, Desc & ".url")
    Exe = "file://" & BuildPath (InstallTarget, WhatsNewFile)
    call CreateUrl (Name, Exe, Desc)

    ' Update the uninstall control file
    call WriteUninstallCtrlFile ("D " & Folder)
end sub



'******************************************************************************
' Function that creates a shortcut on the desktop
'******************************************************************************
sub CreateDesktopIcon ()
    CreateAppShortcut (Desktop)
end sub



'******************************************************************************
' Function that creates the basic INI file
'******************************************************************************
function IniLine (Key, Value)
    IniLine = Key & " = " & Chr (34) & Value & Chr (34)
end function



'******************************************************************************
' Function that makes the INI file writable by everyone
'******************************************************************************
function MakeWorldWritable (Filename)
    dim WinXP

    ' Make a file writable for everyone. Windows XP and lower has cacls,
    ' which doesn't support SIDs. This means that we don't know how the
    ' "Everyone" user is called in the current language (GREAT INNOVATIVE
    ' idea to localize the standard users names), try the ones we know.
    ' Windows VISTA and above have icacls, which supports SIDs but has a
    ' different syntax
    select case WindowsVersion
        case WIN_2000
            WinXP = true
        case WIN_XP
            WinXP = true
        case else
            WinXP = false
    end select
    if (WinXP) then
        ' Try the localized names we know
        call Run ("cacls " & QuoteStr (Filename) & " /E /G Jeder:C", 0)
        call Run ("cacls " & QuoteStr (Filename) & " /E /G Everyone:C", 0)
    else
        call Run ("icacls " & QuoteStr (Filename) & " /grant *S-1-1-0:M", 0)
    end if
end function



'******************************************************************************
' Function that updates an existing INI file
'******************************************************************************
sub UpgradeIniFile ()
    dim SourceName, TargetName
    dim SourceIni, TargetIni
    dim Args(2)
    dim Line, Section
    dim ExePath
    dim HaveV6, HaveV7, HaveV7ng, HaveOptV7ng
    dim Result

    ' Open the existing INI file
    on error resume next
    SourceName = BuildPath (InstallTarget, "s320term.ini")
    set SourceIni = FSO.OpenTextFile (SourceName, ForReading)
    if Err.Number <> 0 then
        Args(1) = SourceName
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_OPENERR", Args))
        exit sub
    end if
    on error goto 0

    ' Create a new INI file
    on error resume next
    TargetName = BuildPath (InstallTarget, "s320term.ini.tmp")
    set TargetIni = FSO.CreateTextFile (TargetName, true)
    if Err.Number <> 0 then
        Args(1) = TargetName
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_CREATEERR", Args))
        exit sub
    end if
    on error goto 0

    ' Read the old INI file copying it over to the new one. Add missing
    ' lines for new compiler versions.
    do while not SourceIni.AtEndOfStream
        ' Read the next line from the file
        on error resume next
        Line = SourceIni.ReadLine
        on error goto 0

        ' Use an additional "Do" to simulate goto/continue
        do
            ' Ignore empty lines
            if Trim (Line) = "" then exit do

            ' Check for start of new section
            if Left (Line, 1) = "[" then
                ' New section starts here
                if Section = "[INCLUDE]" then
                    if not HaveV6 then
                        ExePath = BuildPath (InstallTarget, "spell.6\")
                        TargetIni.WriteLine (IniLine ("Include_V6", ExePath))
                    end if
                    if not HaveV7 then
                        ExePath = BuildPath (InstallTarget, "spell.7\")
                        TargetIni.WriteLine (IniLine ("Include_V7", ExePath))
                    end if
                    if not HaveV7ng then
                        ExePath = BuildPath (InstallTarget, "spell.7\")
                        TargetIni.WriteLine (IniLine ("Include_V7-NG", ExePath))
                    end if
                elseif Section = "[COMPILER]" then
                    if not HaveV6 then
                        ExePath = BuildPath (InstallTarget, "spell.6\ntspc.exe")
                        TargetIni.WriteLine (IniLine ("Compiler_V6", ExePath))
                    end if
                    if not HaveV7 then
                        ExePath = BuildPath (InstallTarget, "spell.7\ntspc.exe")
                        TargetIni.WriteLine (IniLine ("Compiler_V7", ExePath))
                    end if
                    if not HaveV7ng then
                        ExePath = BuildPath (InstallTarget, "spell.7\ntspc.exe")
                        TargetIni.WriteLine (IniLine ("Compiler_V7-NG", ExePath))
                    end if
                    if not HaveOptV7ng then
                        TargetIni.WriteLine (IniLine ("Options_V7-NG", "--ng"))
                    end if                        
                end if
                HaveV6 = false
                HaveV7 = false
                HaveV7ng = false
                HaveOptV7ng = false
                Section = Line

                ' Place an empty line at the end of the section for readbility
                TargetIni.WriteLine ("")
            else
                if Section = "[INCLUDE]" then
                    if Left (Line, 10) = "Include_V6" then 
                        HaveV6 = true
                    elseif Left (Line, 13) = "Include_V7-NG" then 
                        HaveV7ng = true
                    elseif Left (Line, 10) = "Include_V7" then 
                        HaveV7 = true
                    end if
                elseif Section = "[COMPILER]" then
                    if Left (Line, 11) = "Compiler_V6" then 
                        HaveV6 = true
                    elseif Left (Line, 14) = "Compiler_V7-NG" then 
                        HaveV7ng = true
                    elseif Left (Line, 11) = "Compiler_V7" then 
                        HaveV7 = true
                    elseif Left (Line, 13) = "Options_V7-NG" then
                        HaveOptV7ng = true
                    end if                    
                end if
            end if
            TargetIni.WriteLine (Line)
        loop while false
    loop

    ' Beide Dateien schliessen
    SourceIni.Close
    TargetIni.Close

    ' Das Orginal lschen
    Result = DeleteFile (SourceName)
    if Result <> "" then
        Args(1) = SourceName
        Args(2) = Result
        ErrorMsg (FmtMsg ("MSG_DELETEERR", Args))
        exit sub
    end if

    ' Neue Version ber das Orginal kopieren
    Result = MoveFile (TargetName, SourceName)
    if Result <> "" then
        Args(1) = TargetName
        Args(2) = SourceName
        Args(2) = Result
        ErrorMsg (FmtMsg ("MSG_MOVEERR", Args))
        exit sub
    end if

    ' Make the INI file writable for everyone.
    call MakeWorldWritable (SourceName)
end sub



'******************************************************************************
' Function that creates the basic INI file
'******************************************************************************
sub CreateIniFile ()
    dim Filename, Ini, Cmd
    dim WinXP

    ' Create a new INI file
    On Error Resume Next
    Filename = BuildPath (InstallTarget, "s320term.ini")
    set Ini = FSO.CreateTextFile (Filename, true)
    if Err.Number <> 0 then
        dim Args(2)
        Args(1) = Filename
        Args(2) = Err.Description
        call ErrorMsg (FmtMsg ("MSG_CREATEERR", Args))
        exit sub
    end if

    ' Write the default stuff to the INI file
    Ini.WriteBlankLines (1)
    Ini.WriteLine ("[INCLUDE]")
    Ini.WriteLine (IniLine ("Include_V3", BuildPath (InstallTarget, "spell.3\")))
    Ini.WriteLine (IniLine ("Include_V4", BuildPath (InstallTarget, "spell.4\")))
    Ini.WriteLine (IniLine ("Include_V5", BuildPath (InstallTarget, "spell.5\")))
    Ini.WriteLine (IniLine ("Include_V6", BuildPath (InstallTarget, "spell.6\")))
    Ini.WriteLine (IniLine ("Include_V7", BuildPath (InstallTarget, "spell.7\")))
    Ini.WriteLine (IniLine ("Include_V7-NG", BuildPath (InstallTarget, "spell.7\")))
    Ini.WriteBlankLines (1)
    Ini.WriteLine ("[COMPILER]")
    Ini.WriteLine (IniLine ("Compiler_V3", BuildPath (InstallTarget, "spell.3\ntspc.exe")))
    Ini.WriteLine (IniLine ("Compiler_V4", BuildPath (InstallTarget, "spell.4\ntspc.exe")))
    Ini.WriteLine (IniLine ("Compiler_V5", BuildPath (InstallTarget, "spell.5\ntspc.exe")))
    Ini.WriteLine (IniLine ("Compiler_V6", BuildPath (InstallTarget, "spell.6\ntspc.exe")))
    Ini.WriteLine (IniLine ("Compiler_V7", BuildPath (InstallTarget, "spell.7\ntspc.exe")))
    Ini.WriteLine (IniLine ("Compiler_V7-NG", BuildPath (InstallTarget, "spell.7\ntspc.exe")))
    Ini.WriteLine (IniLine ("Options_V7-NG", "--ng"))
    Ini.WriteBlankLines (1)
    Ini.WriteLine ("[MISC]")
    Ini.WriteLine ("DefaultSpellVer = 7")
    Ini.WriteLine ("DefaultHW = 0")
    Ini.WriteLine (IniLine ("RemoteDisplay", BuildPath (TetraTecPrograms, "rd320\rd320.exe")))
    Ini.WriteBlankLines (1)
    Ini.WriteLine ("[MANUAL]")
    If (Language = "de") Then
	Ini.WriteLine ("HelpLanguage = 1")
    Else
	Ini.WriteLine ("HelpLanguage = 0")
    End If

    Ini.Close

    ' Make the INI file writable for everyone.
    call MakeWorldWritable (Filename)
end sub



'******************************************************************************
' Function that tells the user that the install was successful
'******************************************************************************
sub Success ()
    call MsgBox (GetMsg ("MSG_SUCCESS"), vbOkOnly + vbInformation, Installer)
end sub



'******************************************************************************
' Main program
'******************************************************************************
sub Main ()

    ' Initialize global variables. This includes the paths used
    call InitializeGlobals ()
    if Dbg then
        call ShowPathsAndLocations ()
    end if

    ' Check that we're running this script as admin
    call CheckAdminRights ()

    ' Check if there is an old installation and offer to remove it
    call CheckOldInstall ()

    ' Check if the source directory does really exist
    call CheckInstallTarget ()

    ' Display the progress bar
    call ProgressBar (0)

    ' Create the uninstall file
    call CreateUninstallCtrlFile ()
    call ProgressBar (2)

    ' Create registry entries
    CreateRegEntries ()
    call Progressbar (5)

    ' Copy the application files (will do automatic progress bar updates)
    call CopyFiles ()

    ' Copy the uninstaller if necessary
    call CopyUninstaller ()
    call ProgressBar (75)

    ' Create the menu entries
    call CreateMenuEntries ()
    call ProgressBar (80)

    ' Create the desktop icon
    call CreateDesktopIcon ()
    call ProgressBar (90)

    ' Close the uninstall control file
    call CloseUninstallCtrlFile ()

    ' Create a basic INI file if necessary or update an existing one
    if KeepIni then
        call UpgradeIniFile ()
    elseif SharedIni then
        call CreateIniFile ()
    end if

    ' We're done
    call ProgressBar (100)
    call ProgressBar (-1)
    call Success ()

    ' Return a success code
    WScript.Quit (0)
end sub



'******************************************************************************
' The script body just calls Main...
'******************************************************************************
Main ()




